/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "core/copysourcefile.h"

#include <QFile>
#include <QFileInfo>

#include <kdebug.h>

CopySourceFile::CopySourceFile(const QString& filename, qint32 sectorsize) :
	CopySource(),
	m_File(filename),
	m_SectorSize(sectorsize)
{
}

bool CopySourceFile::open()
{
	return file().open(QIODevice::ReadOnly);
}

qint64 CopySourceFile::length() const
{
	return QFileInfo(file()).size() / sectorSize();
}

bool CopySourceFile::readSectors(void* buffer, qint64 readOffset, qint64 numSectors)
{
	if (!file().seek(readOffset * sectorSize()))
		return false;

	return file().read(static_cast<char*>(buffer), numSectors * sectorSize()) == numSectors * sectorSize();
}
