/***************************************************************************
 *   Copyright (C) 2008 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "gui/partwidgetbase.h"
#include "gui/partwidget.h"

#include "core/partition.h"

#include <kdebug.h>

const int PartWidgetBase::m_Spacing = 3;
const int PartWidgetBase::m_BorderWidth = 4;
const int PartWidgetBase::m_BorderHeight = 6;

void PartWidgetBase::positionChildren(const QWidget* destWidget, const PartitionNode::Partitions& partitions, QList<PartWidget*>& widgets)
{
	// we need to do some recalculations before we can draw the widgets with correct
	// sizes because if one or more partitions are too small to be drawn reasonably,
	// we need to reduce the other partition's sizes to draw those a little wider

	// the minimum width
	const int minWidth = 20;

	// an array of all widths
	QList<qint64> childrenWidth;

	// the total capacity
	qint64 totalCapacity = 0;

	// first, go through the partitions, remember the capacity of each of them
	// and calculate the total capacity
	foreach(const Partition* p, partitions)
	{
		childrenWidth.append(p->capacity());
		totalCapacity += p->capacity();
	}

	// now go though the children again and calculate the on screen width for each
	// of them. if we find out a partition is too small, adjust its screen
	// width to minWidth and increase adjust by how much we had to increase the
	// screen width. thus, in the end, we have the number of pixels we need
	// to find somewhere else in adjust. and the number of adjustments goes in
	// numAdjusted.
	int adjust = 0;
	int numAdjusted = 0;
	for (int i = 0; i < childrenWidth.size(); i++)
	{
		const qint64 w = childrenWidth[i] * (destWidget->width() - 2 * PartWidget::borderWidth())  / totalCapacity;

		childrenWidth[i] = w;
		if (w < minWidth)
		{
			adjust += minWidth - w;
			childrenWidth[i] = minWidth;
			numAdjusted++;
		}
	}

	// if we have adjusted one or more partitions (and not ALL of them, because in that
	// case, nothing will help us), go through the partitions again and reduce the
	// on screen widths of those big enough anyway
	if (adjust > 0 && numAdjusted < childrenWidth.size())
	{
		const int reduce = adjust / (childrenWidth.size() - numAdjusted);

		for (int i = 0; i < childrenWidth.size(); i++)
			if (childrenWidth[i] > minWidth + reduce)
				childrenWidth[i] -= reduce;
	}

	// finally, move the children to their positions and resize them
	for (int i = 0, x = PartWidget::borderWidth(); i < widgets.size(); i++)
	{
		widgets[i]->move(x, PartWidget::borderHeight());
		widgets[i]->resize(childrenWidth[i] - PartWidget::spacing(), destWidget->height() - 2 * PartWidget::borderHeight());
		x += childrenWidth[i];
	}
}

